@@ -56,10 +56,10 @@ class SmartDriver(SeleniumDriverCore):
             self._test_case_upload_screenshot(element_name)
             element_box, needs_reload, has_training_data = self._test_case_get_box(element_name)
             if element_box:
-                real_elem = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier)
-                #element = ai_elem(real_elem.parent, real_elem, element_box, self.driver,
-                #                  self.multiplier)
-                element = real_elem
+                element = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier, use_first=self.use_ai_elem)
+                if self.use_ai_elem:
+                    element = ai_elem(None, element._id, element_box, self.driver,
+                                      self.multiplier, smart_driver=self, base_elem=real_elem)
                 return element, self.last_test_case_screenshot_uuid, msg
             else:
                 event_id = str(uuid.uuid4())
@@ -71,8 +71,9 @@ class SmartDriver(SeleniumDriverCore):
                     element_box, needs_reload, has_training_data = self._test_case_get_box(element_name, event_id=event_id)
                     if element_box is not None:
                         print('Element was labeled, moving on')
-                        real_elem = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier)
-                        element = ai_elem(real_elem.parent, real_elem._id, element_box, self.driver,
+                        element = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier, use_first=self.use_ai_elem)
+                        if self.use_ai_elem:
+                            element = ai_elem(None, element._id, element_box, self.driver,
                                           self.multiplier, smart_driver=self, base_elem=real_elem)
                         return element, self.last_test_case_screenshot_uuid, msg
 
@@ -91,10 +92,10 @@ class SmartDriver(SeleniumDriverCore):
             if resp_data['success'] and 'box' in resp_data:
                 if self.debug:
                     print(f'Found cached box in action info for {element_name} using that')
-                real_elem = self._match_bounding_box_to_web_element(resp_data['box'], multiplier=self.multiplier)
-                #element = ai_elem(real_elem.parent, real_elem, resp_data['box'], self.driver,
-                #                  self.multiplier)
-                element = real_elem
+                element = self._match_bounding_box_to_web_element(resp_data['box'], multiplier=self.multiplier, use_first=self.use_ai_elem)
+                if self.use_ai_elem:
+                    element = ai_elem(real_elem.parent, real_elem, resp_data['box'], self.driver,
+                                    self.multiplier)
                 return element, key, msg
 
             source = ''
@@ -122,24 +123,14 @@ class SmartDriver(SeleniumDriverCore):
                 msg = msg.replace(self.default_prod_url, self.url)
 
                 element_box = response['predicted_element']
+                element = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier, use_first=self.use_ai_elem)
                 if self.use_ai_elem:
-                    parent_elem = None
-                    real_elem = element_box
-                    if self.root_elem is None:
-                        self.root_elem =  self.driver.find_elements(by='xpath', value='//*')[0]._id
-
-                    elem_id = self.root_elem
-                else:
-                    real_elem = self._match_bounding_box_to_web_element(
-                        element_box, multiplier=self.multiplier)
-                    parent_elem = real_elem.parent
-                    elem_id = real_elem._id
-                element = ai_elem(parent_elem, elem_id, element_box, self.driver, self.multiplier, smart_driver=self, base_elem=real_elem)
+                    element = ai_elem(None, elem_id, element_box, self.driver, self.multiplier, smart_driver=self, base_elem=real_elem)
             except Exception:
                 logging.exception('exception during classification')
             return element, run_key, msg
 
-    def _match_bounding_box_to_web_element(self, bounding_box, multiplier=1):
+    def _match_bounding_box_to_web_element(self, bounding_box, multiplier=1, use_first=False):
         """
             We have to ba hacky about this becasue Selenium does not let us click by coordinates.
             We retrieve all elements, compute the IOU between the bounding_box and all the elements and pick the best match.
@@ -154,6 +145,8 @@ class SmartDriver(SeleniumDriverCore):
             elements = self.driver.find_elements(by='xpath', value='//*')
         except StaleElementReferenceException:
             elements = self.driver.find_elements(by='xpath', value='//*')
+        if use_first:
+            return elements[0]
 
         # Compute IOU
         iou_scores = []
