@@ -31,7 +31,7 @@ from selenium.common.exceptions import StaleElementReferenceException
 from . import __version__ as base_version
 from .utils.selenium_core import SeleniumDriverCore
 from .utils.local_classify import LocalClassifier
-
+from .utils.misc import collect_stack_trace
 
 requests.packages.urllib3.disable_warnings()
 
@@ -151,8 +151,10 @@ class SmartDriver(SeleniumDriverCore):
             log.debug(e)
         return element, key, msg
 
-    def _classify(self, element_name, is_backup=True):
+    def _classify(self, element_name, is_backup=True, custom_ai_threshold=None):
         msg = ''
+        stack_trace = collect_stack_trace()
+
         if self.test_case_creation_mode:
             if self.do_local_caching:
                 screenshotBase64 = self._get_screenshot()
@@ -176,7 +178,7 @@ class SmartDriver(SeleniumDriverCore):
                 return element, self.last_test_case_screenshot_uuid, msg
             else:
                 if has_training_data:
-                    response = self._classify_full_screen(element_name)
+                    response = self._classify_full_screen(element_name, custom_ai_threshold=custom_ai_threshold)
                     if response is not None and response['success']:
                         element_box = response['predicted_element']
                         self.page_offset = self.driver.execute_script('return window.pageYOffset')
@@ -192,7 +194,7 @@ class SmartDriver(SeleniumDriverCore):
                                                                                                        label_url))
                 webbrowser.open(label_url)
                 while True:
-                    element_box, needs_reload, _ = self._test_case_get_box(element_name, event_id=event_id)
+                    element_box, needs_reload, _ = self._test_case_get_box(element_name, event_id=event_id, custom_ai_threshold=custom_ai_threshold)
                     if element_box is not None:
                         element = self._get_matched_elem(element_box)
                         return element, self.last_test_case_screenshot_uuid, msg
@@ -247,6 +249,8 @@ class SmartDriver(SeleniumDriverCore):
                         'test_case_name': self.test_case_uuid,
                         'do_exact_match_first': self.do_exact_match_first,
                         'exact_match_first_threshold': self.exact_match_first_threshold,
+                        'stack_trace': stack_trace,
+                        'custom_ai_threshold': custom_ai_threshold,
                         }
                 start = time.time()
                 response = self.make_json_post_request('/detect', data, self.detect_timeout_message, self.detect_timeout)
@@ -255,7 +259,7 @@ class SmartDriver(SeleniumDriverCore):
                     print(f'Classify time: {end - start}')
                 if not response['success']:
                     # Could be a failure or an element off screen
-                    response = self._classify_full_screen(element_name)
+                    response = self._classify_full_screen(element_name, custom_ai_threshold=custom_ai_threshold)
                     if not response['success']:
                         classification_error_msg = response['message'].replace(self.default_prod_url, self.url)
                         log.debug(classification_error_msg)
@@ -280,7 +284,7 @@ class SmartDriver(SeleniumDriverCore):
                 logging.exception('exception during classification')
             return element, run_key, msg
 
-    def _classify_full_screen(self, element_name):
+    def _classify_full_screen(self, element_name, custom_ai_threshold=None):
         self._scroll_page(-100000) # go to the top
         last_offset = -1
         offset = 1
@@ -290,13 +294,16 @@ class SmartDriver(SeleniumDriverCore):
         while offset > last_offset:
             last_offset = offset
             screenshotBase64 = self._get_screenshot()
+            stack_trace = collect_stack_trace()
             data = {'screenshot': screenshotBase64,
                     'source': '',
                     'api_key': self.api_key,
                     'label': element_name,
                     'test_case_name': self.test_case_uuid,
                     'do_exact_match_first': self.do_exact_match_first,
-                    'exact_match_first_threshold': self.exact_match_first_threshold,}
+                    'exact_match_first_threshold': self.exact_match_first_threshold,
+                    'stack_trace': stack_trace,
+                    'custom_ai_threshold': custom_ai_threshold}
             r = self.make_json_post_request('/detect', data, self.detect_timeout_message, self.detect_timeout)
             if r['success']:
                 return r
@@ -346,7 +353,9 @@ class SmartDriver(SeleniumDriverCore):
             We retrieve all elements, compute the IOU between the bounding_box and all the elements and pick the best match.
         """
         if self.use_fast_js_chopper:
-            return self._match_bounding_box_to_selenium_element_js(bounding_box, multiplier=multiplier)
+            res = self._match_bounding_box_to_selenium_element_js(bounding_box, multiplier=multiplier)
+            if res is not None:
+                return res
 
         # Adapt box to local coordinates
         multiplier = max(1, multiplier) # Make sure multiplier isn't 0.
@@ -376,7 +385,7 @@ class SmartDriver(SeleniumDriverCore):
             they are a valid candidate. If none of them is of type input, we pick the one with maxIOU, otherwise we pick the input type,
             which is 90% of test cases.
             """
-            composite = filter(lambda x: x[0] > 0.2, composite)
+            composite = filter(lambda x: x[0] > 0.01, composite)
             composite = list(filter(lambda x: self._center_hit(new_box, x[1].rect), composite))
             end = time.time()
 
@@ -507,4 +516,8 @@ class NoElementFoundException(Exception):
 
 from selenium.webdriver.common.by import By
 class By(By):
-    AI = 'ai'
\ No newline at end of file
+    AI = 'ai'
+
+    @staticmethod
+    def ai(elementName, custom_ai_threshold=None):
+        return ("ai", elementName, custom_ai_threshold)
\ No newline at end of file
